package sliders;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;


public class Sliders extends JPanel implements
  ActionListener, ChangeListener
{
  JPanel      panel;
  JSlider     simpleSlider, tickSlider, snapSlider;
  JSlider     labelSlider, customSlider;
  Dictionary  labelTable;
  ImageIcon   icon[] = new ImageIcon[2];
  JLabel      label;
  JCheckBox   invertedCheck;
  JLabel      sliderLabel;


  Sliders(JApplet applet)
  {
    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

    // Einfacher Slider
    simpleSlider = new JSlider(JSlider.HORIZONTAL, 0, 100, 30);
    simpleSlider.setBorder(BorderFactory.createTitledBorder(
      new EtchedBorder(), "Einfacher Slider", 0, 0,
      new Font("Dialog", Font.BOLD, 12)));
    simpleSlider.addChangeListener(this);
    add(simpleSlider);

    // Mit Marken
    tickSlider = new JSlider(JSlider.HORIZONTAL, 0, 100, 70);
    tickSlider.setBorder(BorderFactory.createTitledBorder(
      new EtchedBorder(), "Mit Marken", 0, 0,
      new Font("Dialog", Font.BOLD, 12)));
    tickSlider.addChangeListener(this);
    tickSlider.setPaintTicks(true);
    tickSlider.setMajorTickSpacing(10);
    tickSlider.setMinorTickSpacing(5);
    add(tickSlider);

    // Mit Rasterung
    snapSlider = new JSlider(JSlider.HORIZONTAL, 0, 10, 5);
    snapSlider.setBorder(BorderFactory.createTitledBorder(
      new EtchedBorder(), "Mit Rasterung", 0, 0,
      new Font("Dialog", Font.BOLD, 12)));
    snapSlider.addChangeListener(this);
    snapSlider.setPaintTicks(true);
    snapSlider.setMajorTickSpacing(1);
    snapSlider.setSnapToTicks(true);
    add(snapSlider);

    // Mit Beschriftung
    labelSlider = new JSlider(JSlider.HORIZONTAL, 0, 100, 80);
    labelSlider.setBorder(BorderFactory.createTitledBorder(
      new EtchedBorder(), "Mit Beschriftung und Icons", 0, 0,
      new Font("Dialog", Font.BOLD, 12)));
    labelSlider.addChangeListener(this);
    labelSlider.setPaintTicks(true);
    labelSlider.setMajorTickSpacing(10);
    labelSlider.setMinorTickSpacing(1);
    labelSlider.setSnapToTicks(true);
    labelSlider.setPaintLabels(true);
    add(labelSlider);

    // Mit Spezialbeschriftung
    try
    {
      icon[0] = new ImageIcon("ok.gif");
      icon[1] = new ImageIcon("danger.gif");
    }
    catch(SecurityException se)      // Fr Internet-Browser
    {
      icon[0] = new ImageIcon(
        applet.getImage(applet.getCodeBase(), "ok.gif"));
      icon[1] = new ImageIcon(
        applet.getImage(applet.getCodeBase(), "danger.gif"));
    }
    customSlider = new JSlider(JSlider.HORIZONTAL, 0, 10, 2);
    customSlider.setBorder(BorderFactory.createTitledBorder(
      new EtchedBorder(), "Mit Beschriftung", 0, 0,
      new Font("Dialog", Font.BOLD, 12)));
    customSlider.addChangeListener(this);
    customSlider.setPaintTicks(true);
    customSlider.setMajorTickSpacing(5);
    customSlider.setMinorTickSpacing(1);
    customSlider.setSnapToTicks(true);
    customSlider.setPaintLabels(true);
    labelTable = customSlider.getLabelTable();
    labelTable.put(new Integer(3), label =
      new JLabel("3 optimal", icon[0], JLabel.CENTER));
    labelTable.put(new Integer(8), label =
      new JLabel("8 Gefahr", icon[1], JLabel.CENTER));
    customSlider.setLabelTable(labelTable);
    add(customSlider);

    // Option: Invertiert
    add(panel = new JPanel());
    panel.setLayout(new GridLayout(0, 1, 10, 5));
    invertedCheck = new JCheckBox("Invertiert");
    invertedCheck.addActionListener(this);
    panel.add(invertedCheck);

    // Sliderwert
    sliderLabel = new JLabel("Sliderwert: " + 0);
    panel.add(sliderLabel);
  }


  // Erstellt ein Panel mit Rahmen
  public static JPanel createTitledPanel(String title)
  {
    JPanel p = new JPanel();
    p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
    p.setBorder(BorderFactory.createTitledBorder(
      new EtchedBorder(), title, 0, 0,
      new Font("Dialog", Font.BOLD, 12)));
    return p;
  }


  public void actionPerformed(ActionEvent e)
  {
    if (e.getActionCommand() == "Invertiert")
    {
      if (invertedCheck.isSelected())
      {
        simpleSlider.setInverted(true);
        tickSlider.setInverted(true);
        snapSlider.setInverted(true);
        labelSlider.setInverted(true);
        customSlider.setInverted(true);
      }
      else
      {
        simpleSlider.setInverted(false);
        tickSlider.setInverted(false);
        snapSlider.setInverted(false);
        labelSlider.setInverted(false);
        customSlider.setInverted(false);
      }
      repaint();
    }

  }

  public void stateChanged(ChangeEvent e)
  {
    JSlider s;
    s = (JSlider)e.getSource();
    sliderLabel.setText("Sliderwert: " + s.getValue());
  }
}