package optionpane;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;


public class OptionPane extends JPanel implements ActionListener
{
  JPanel       dialogButtonPanel; // Panel fr die 4 Dialog-Buttons
  JPanel       optionPanel;       // fr Optionen
  JPanel       messagePanel;      // Message-Typen
  JPanel       resultPanel;       // Dialogergebnisse
  JPanel       customPanel;
  JOptionPane  optionPane;
  JTextField   confirmField, inputField, optionField;
  ButtonGroup  optionGroup;
  ButtonGroup  messageGroup;
  JButton      button;
  JRadioButton radioButton;
  JCheckBox    customIconCheck, inputCheck;
  ImageIcon    dukeIcon;
  ImageIcon    customIcon;

  String  inputValues[] =
          {"1. Mglichkeit: rechts ", "2. Mglichkeit: links",
           "3. Mglichkeit: vorne"};
  String  messageNames[] =
          {"ERROR_MESSAGE", "INFORMATION_MESSAGE",
           "WARNING_MESSAGE", "QUESTION_MESSAGE", "PLAIN_MESSAGE"};
  int  messageValues[] =
       {JOptionPane.ERROR_MESSAGE, JOptionPane.INFORMATION_MESSAGE,
        JOptionPane.WARNING_MESSAGE, JOptionPane.QUESTION_MESSAGE,
        JOptionPane.PLAIN_MESSAGE};
  String  optionNames[] =
        {"DEFAULT_OPTION", "YES_NO_OPTION", "YES_NO_CANCEL_OPTION",
         "OK_CANCEL_OPTION"};
  int optionValues[] =
      {JOptionPane.DEFAULT_OPTION, JOptionPane.YES_NO_OPTION,
    JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.OK_CANCEL_OPTION};
  int     dialogOption = optionValues[0];
  int     messageType  = messageValues[0];


  OptionPane(JApplet applet)
  {
    setLayout(null);
    try
    {
      dukeIcon = new ImageIcon("duke.gif");
    }
    catch(SecurityException se)  // Fr Internet-Browser
    {
      dukeIcon = new ImageIcon(
        applet.getImage(applet.getCodeBase(), "duke.gif"));
    }
    optionPane = new JOptionPane();
    optionPane.setInputValue("Test");

    // Buttons fr die vier Dialogarten
    dialogButtonPanel = createTitledPanel("Standard-Dialoge",
      10, 10, 200, 230);
    dialogButtonPanel.add(button = new JButton("ConfirmDialog"));
    button.addActionListener(this);
    dialogButtonPanel.add(button = new JButton("InputDialog"));
    button.addActionListener(this);
    dialogButtonPanel.add(button = new JButton("MessageDialog"));
    button.addActionListener(this);
    dialogButtonPanel.add(button = new JButton("OptionDialog"));
    button.addActionListener(this);
    add(dialogButtonPanel);

    // Optionen fr die Dialoge
    optionPanel = createTitledPanel("Dialogoptionen",
      240, 10, 210, 100);
    optionGroup = new ButtonGroup();
    for (int i=0; i<4; i++)
    {
      optionPanel.add(radioButton = new JRadioButton(optionNames[i]));
      radioButton.setSelected(i == 0);  // 1. Button selektiert
      radioButton.addActionListener(this);
      optionGroup.add(radioButton);
    }
    add(optionPanel);

    // Message-Typen fr die Dialoge
    messagePanel = createTitledPanel("Message-Typen",
      240, 120, 210, 120);
    messageGroup = new ButtonGroup();
    for (int i=0; i<5; i++)
    {
      messagePanel.add(radioButton = new JRadioButton(messageNames[i]));
      radioButton.setSelected(i == 0);  // 1. Button selektiert
      radioButton.addActionListener(this);
      messageGroup.add(radioButton);
    }
    add(messagePanel);

    // Sonstige Optionen
    customIconCheck = new JCheckBox("Mit eigenem Icon");
    customIconCheck.setBounds(20, 240, 130, 20);
    add(customIconCheck);
    inputCheck = new JCheckBox(
      "Mit Auswahlmglichkeit beim InputDialog und OptionDialog");
    inputCheck.setBounds(20, 260, 440, 20);
    add(inputCheck);

    // Ergebnisse der Dialoge
    resultPanel = createTitledPanel("Ergebnisse",
      10, 290, 440, 100);
    resultPanel.setLayout(new GridLayout(0, 2, 10, 5));
    resultPanel.add(new JLabel("ConfirmDialog"));
    resultPanel.add(confirmField = new JTextField());
    resultPanel.add(new JLabel("InputDialog"));
    resultPanel.add(inputField = new JTextField());
    resultPanel.add(new JLabel("OptionDialog"));
    resultPanel.add(optionField = new JTextField());
    add(resultPanel);
  }


  public void actionPerformed(ActionEvent e)
  {
    int       optionValue=0;    // gewhlte Dialogoption
    String    inputText=null;   // Eingabetext fr den InputDialog

    // Mit benutzerdefinierten Icon ?
    if (customIconCheck.isSelected())
      customIcon = dukeIcon;  // Ja
    else
      customIcon = null;      // Nein

    // Eine Dialogoption oder ein Message-Typ wurde ausgewhlt
    if (e.getSource() instanceof JRadioButton)
    {
      for (int i=0; i<4; i++)   // Dialogoption
        if (e.getActionCommand() == optionNames[i])
          // setze neue Dialogoption
          dialogOption = optionValues[i];

      for (int i=0; i<5; i++)   // MessageTyp
        if (e.getActionCommand() == messageNames[i])
          // setze neuen MessageTyp
          messageType = messageValues[i];
    }

    // Ein Dialogtyp wurde gewhlt
    if (e.getSource() instanceof JButton)
    {
      if (e.getActionCommand() == "ConfirmDialog")
      {
        optionValue = optionPane.showConfirmDialog(this,
          "Hallo! Das ist ein ConfirmDialog.", "ConfirmDialog",
          dialogOption, messageType, customIcon);
        // Ergebnis ins Ergebnisfeld schreiben
        confirmField.setText(Integer.toString(optionValue));
      }

      if (e.getActionCommand() == "InputDialog")
      {
        if (inputCheck.isSelected())
          // Mit ComboBox
          inputText = (String)optionPane.showInputDialog(this,
            "Hallo! Das ist ein InputDialog.", "InputDialog",
            messageType, customIcon, inputValues, inputValues[0]);
        else
          // Mit Textfeld
          inputText = (String)optionPane.showInputDialog(this,
            "Hallo! Das ist ein InputDialog.", "InputDialog",
            messageType, customIcon, null, null);
         // Ergebnis ins Ergebnisfeld schreiben
        inputField.setText(inputText);
      }

      // Einfacher MessageDialog
      if (e.getActionCommand() == "MessageDialog")
        optionPane.showMessageDialog(this,
          "Hallo! Das ist ein MessageDialog.",
          "MessageDialog", messageType, customIcon);

      // OptionDialog
      if (e.getActionCommand() == "OptionDialog")
      {
        if (inputCheck.isSelected())
          optionValue = optionPane.showOptionDialog(this,
            "Hallo! Das ist ein OptionDialog.", "OptionDialog",
            dialogOption, messageType, customIcon, inputValues,
            inputValues[0]); // Mit Optionen
        else
          optionValue = optionPane.showOptionDialog(this,
            "Hallo! Das ist ein OptionDialog.", "OptionDialog",
            dialogOption, messageType, customIcon, null, null);
        optionField.setText(Integer.toString(optionValue));
      }
    }
  }


  // Erstellt ein Panel mit Rahmen
  public static JPanel createTitledPanel(String title, int x, int y,
    int width, int height)
  {
    JPanel p = new JPanel();
    p.setLayout(new GridLayout(0, 1, 30, 5));
    p.setBorder(BorderFactory.createTitledBorder(
      new EtchedBorder(), title, 0, 0,
      new Font("Dialog", Font.BOLD, 12)));
    p.setBounds(x, y, width, height);
    return p;
  }
}