package splitpane;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import javax.swing.tree.*;


public class SplitPane extends JPanel implements
  TreeSelectionListener
{
  JSplitPane     splitPane;
  JTree          simpleTree;
  DefaultMutableTreeNode  javaRoot;
  DefaultMutableTreeNode  classNode;
  JScrollPane    scrollPane;
  JTextArea     textArea;

  String    AWTNames[] =
  {"Button", "Canvas", "CardLayout", "Checkbox", "CheckboxGroup",
   "CheckboxMenuItem", "Choice", "Color", "Component",
   "Container", "Cursor", "Dialog", "Dimension", "Event",
   "EventQueue", "FileDialog", "FlowLayout", "Font",
   "FontMetrics", "Frame", "Graphics", "GridBagConstraints",
   "GridBagLayout", "GridLayout", "Image", "Insets", "Label",
   "etc."};
  String    JFCNames[] =
  {"JCheckBox", "JCheckBoxMenuItem", "JComboBox", "JComponent",
   "JDesktopPane", "JDialog", "JEditorPane", "JFrame",
   "JInternalFrame", "JLabel", "JLayeredPane", "JList","JMenu",
   "JMenuBar", "JMenuItem", "JOptionPane", "JPanel",
   "JPasswordField", "JPopupMenu", "JProgressBar","JRadioButton",
   "JRadioButtonMenuItem", "JRootPane", "JScrollBar",
   "JScrollPane", "JSeparator", "JSlider", "JSplitPane",
   "JTabbedPane", "JTable", "JTextArea", "JTextField", "etc."};


  SplitPane(JApplet applet)
  {
    setLayout(new BorderLayout());

    splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
      false);
    add("Center", splitPane);

    // Auf der linken Seite befindet sich der Klassenbaum
    javaRoot = new DefaultMutableTreeNode("Java Classes");
    javaRoot.add(classNode = new DefaultMutableTreeNode("AWT"));
    for (int i=0; i<28; i++)
      classNode.add(new DefaultMutableTreeNode(AWTNames[i]));
    javaRoot.add(classNode = new DefaultMutableTreeNode("JFC"));
    for (int i=0; i<33; i++)
      classNode.add(new DefaultMutableTreeNode(JFCNames[i]));
    javaRoot.add(new DefaultMutableTreeNode("IFC", false));
    simpleTree = new JTree(javaRoot);
    simpleTree.addTreeSelectionListener(this);
    scrollPane = new JScrollPane(simpleTree);
    splitPane.setLeftComponent(scrollPane);

    // Auf der rechten Seite befindet sich ein Textfeld
    textArea = new JTextArea("");
    textArea.setEditable(false);
    textArea.setFont(new Font("Courier", Font.BOLD, 16));
    splitPane.setRightComponent(textArea);
    splitPane.setOneTouchExpandable(true);

    splitPane.setDividerLocation(0.4);
  }


  public void valueChanged(TreeSelectionEvent e)
  {
    TreePath p = e.getPath();

    if (e.getPath().getPathCount() == 3)
    {
      textArea.setText("Klasse:  " +
        p.getLastPathComponent().toString()
        + "\n\n" + "Hier kann die Dokumentation stehen.");
    }
    else
      textArea.setText("");
  }
}