package comboboxandlist;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;


public class ComboBoxAndList extends JPanel
{
  JApplet         applet;
  JComboBox       comboBox;
  JList           listBox;
  JScrollPane     scrollPane;
  MyListModel     myListModel;
  MyCellRenderer  myCellRenderer;


  ComboBoxAndList(JApplet a)
  {
    applet = a;

    setLayout(new BoxLayout(this, BoxLayout.X_AXIS));

    myListModel  = new MyListModel(applet);
    myCellRenderer = new MyCellRenderer();

    comboBox = new JComboBox(myListModel);
    comboBox.setRenderer(myCellRenderer);
    comboBox.setBorder(BorderFactory.createTitledBorder(new EtchedBorder(), "JComboBox mit MVC"));
    comboBox.setSelectedIndex(1);
    add(comboBox);

    listBox = new JList(myListModel);
    listBox.setCellRenderer(myCellRenderer);
    listBox.setSelectedIndex(1);
    scrollPane = new JScrollPane(listBox);
    scrollPane.setBorder(BorderFactory.createTitledBorder(new EtchedBorder(), "JList mit MVC"));
    add(scrollPane);
  }
}


class MyListModel extends AbstractListModel implements ComboBoxModel
{
  JApplet     applet;         // Verbindung zum Applet
  Object      currentValue;
  String      fileNames[] =   // Namen der Bilddateien
                {"portugieser.gif", "riesling.gif", "rivaner.gif",
                 "rulaender.gif", "scheurebe.gif", "spaetlese.gif"};
  String      weinNames[] =   // Namen der Weinsorten
                {"Portugieser", "Riesling", "Rivaner",
                 "Rulnder", "Scheurebe", "Sptlese"};
  String      weinPreise[] =  // Preise der Weinsorten
                {"20.95 CHF", "18.50 CHF", "22.90 CHF",
                 "16.85 CHF", "22.95 CHF", "25.90 CHF"};
  ImageIcon   weinIcons[] = new ImageIcon[weinNames.length];
  // Listeneintrge
  Hashtable   modelCache[] = new Hashtable[getSize()];


  public MyListModel(JApplet a)
  {
    this.applet = a;

    try
    {
      for (int i=0; i<weinNames.length; i++)  // Icons einladen
        weinIcons[i] = new ImageIcon(fileNames[i]);
    }
    catch(SecurityException se)      // Fr Internet-Browser
    {
      for (int i=0; i<weinNames.length; i++)  // Icons einladen
        weinIcons[i] = new ImageIcon(
          applet.getImage(applet.getCodeBase(), fileNames[i]));
    }
  }

  // Setzt den neuen Wert der Listenelements
  public void setSelectedItem(Object anObject)
  {
    currentValue = anObject;
    fireContentsChanged(this,-1,-1);
  }

  // Liefert den aktuellen wert zurck
  public Object getSelectedItem()
  {
    return currentValue;
  }

  // Liefert die Anzahl der Elemente
  public int getSize()
  {
    return weinNames.length;
  }

  public Object getElementAt(int index)
  {
    if(modelCache[index] != null)
      return modelCache[index];
    else
    {
      Hashtable result = new Hashtable();
      // Text des Elementes im unselektierten Zustand
      result.put("title",  weinNames[index]);
      // Text des Elementes im selektierten Zustand
      result.put("selectedTitle", weinPreise[index]);
      // Icon des Elementes
      result.put("image", weinIcons[index]);
      modelCache[index] = result;
      return result;
    }
  }
}


class MyCellRenderer extends JLabel implements ListCellRenderer
{
  public MyCellRenderer()
  {
    setOpaque(true);
  }

  public Component getListCellRendererComponent(JList listbox,
        Object value, int index, boolean isSelected, boolean cellHasFocus)
  {
    Hashtable h = (Hashtable) value;  // Hashtabelle eines Listenelements

    setOpaque(true);

    if(value == null)
    {
      setText("");
      setIcon(null);
    }
    else
    {
      if(isSelected)
      {  // selektiertes Listenelement
        setBackground(UIManager.getColor("ComboBox.selectedBackground"));
        setForeground(UIManager.getColor("ComboBox.selectedForeground"));
        setHorizontalTextPosition(JLabel.RIGHT);
        setIcon((ImageIcon)h.get("image"));
         // Namen der Weinsort + Preis
        setText((String)h.get("title") + ":  " +
          (String)h.get("selectedTitle"));
      }
      else
      { // unselektiertes Listenelement
        setIcon((ImageIcon)h.get("image"));
        setText((String)h.get("title"));
        setHorizontalTextPosition(JLabel.RIGHT);
        setBackground(UIManager.getColor("ComboBox.background"));
        setForeground(UIManager.getColor("ComboBox.foreground"));
      }
    }
    return this;
  }
}