package eigeneslook;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.*;
import java.io.Serializable;

import javax.swing.plaf.ButtonUI;

public class MyButtonUI extends ButtonUI
   implements Serializable {

 protected final static Insets defaultMargin =
     new Insets (2, 5, 2, 5);
 protected final static Font defaultFont =
     new Font ("Serif", Font.BOLD, 10);

 private final static Border defaultBorder =
   new CompoundBorder(
     MyButtonBorder.getButtonBorder(),
     new BasicBorders.MarginBorder());

 protected static final int textIconGap = 3;

 protected MyButtonListener listener;
 protected static ButtonUI buttonUI;

 public static ComponentUI createUI (JComponent c) {
   if (buttonUI == null) {
     buttonUI = new MyButtonUI();
   }
   return buttonUI;
 }

 public void installUI (JComponent c) {
   listener = new MyButtonListener(c);
   c.addMouseListener(listener);
   c.addMouseMotionListener(listener);
   c.setFont (defaultFont);
   if (c.getBorder() == null)
     c.setBorder (defaultBorder);
 }

 public void uninstallUI (JComponent c) {
   c.removeMouseListener (listener);
   c.removeMouseMotionListener (listener);
   if (c.getBorder() == defaultBorder)
     c.setBorder(null);
 }

 public void paint (Graphics g, JComponent c) {
   AbstractButton ab = (AbstractButton) c;
   ButtonModel bm = ab.getModel();

   Dimension size = ab.getSize();
   g.setFont (c.getFont());
   FontMetrics fm = g.getFontMetrics();

   int shiftOffset = 0;

   Rectangle viewRect = new Rectangle(size);
   Rectangle iconRect = new Rectangle();
   Rectangle textRect = new Rectangle();

   String text = SwingUtilities.layoutCompoundLabel (
     fm, ab.getText(), ab.getIcon(),
     ab.getVerticalAlignment(),
     ab.getHorizontalAlignment(),
     ab.getVerticalTextPosition(),
     ab.getHorizontalTextPosition(),
     viewRect, iconRect, textRect, textIconGap);

   if (bm.isArmed() && bm.isPressed()) {
     shiftOffset = 1;
   }

   // Paint background
   if (c.isOpaque()) {
     g.setColor (ab.getBackground());
     g.fillRect (0, 0, size.width, size.height);
   }


   // Draw Icon

   if (ab.getIcon() != null) {
     Icon icon = null;
     if (!bm.isEnabled()) {
       icon = ab.getDisabledIcon();
     } else if (bm.isPressed() && bm.isArmed()) {
       icon = ab.getPressedIcon();
     } else if (bm.isRollover()) {
       icon = ab.getRolloverIcon();
     }

     if (icon == null) {
       icon = ab.getIcon();
     }

     if (bm.isPressed() && bm.isArmed()) {
       icon.paintIcon (c, g, iconRect.x + shiftOffset,
         iconRect.y + shiftOffset);
     } else {
       icon.paintIcon (c, g, iconRect.x, iconRect.y);
     }
   }

   // Draw Text

   if ((text != null) && (text.length() != 0)) {
     if (bm.isEnabled()) {
       g.setColor (ab.getForeground());
       BasicGraphicsUtils.drawString (g, text,
        // bm.getKeyAccelerator(),
       	0,
         textRect.x + shiftOffset,
         textRect.y + fm.getAscent() + shiftOffset);
     } else {
       g.setColor (ab.getBackground().brighter());
       BasicGraphicsUtils.drawString (g, text,
         // bm.getKeyAccelerator(),
         0,
         textRect.x, textRect.y + fm.getAscent());
       g.setColor (ab.getBackground().darker());
       BasicGraphicsUtils.drawString (g, text,
         // bm.getKeyAccelerator(),
       	0,
         textRect.x - 1, textRect.y + fm.getAscent() - 1);
     }
   }
 }

 public Dimension getMinimumSize (JComponent c) {
   return getPreferredSize (c);
 }

 public Dimension getMaximumSize (JComponent c) {
   return getPreferredSize (c);
 }

 public Dimension getPreferredSize (JComponent c) {
   if ((c.getComponentCount() > 0) ||
       !(c instanceof AbstractButton)) {
     return null;
   }

   AbstractButton ab = (AbstractButton) c;
   Icon icon = ab.getIcon();
   String text = ab.getText();

   Font font = ab.getFont();
   FontMetrics fm = ab.getToolkit().getFontMetrics (font);

   Rectangle viewRect = new Rectangle (
     Short.MAX_VALUE, Short.MAX_VALUE);
   Rectangle iconRect = new Rectangle();
   Rectangle textRect = new Rectangle();

   SwingUtilities.layoutCompoundLabel (
     fm, text, icon,
     ab.getVerticalAlignment(),
     ab.getHorizontalAlignment(),
     ab.getVerticalTextPosition(),
     ab.getHorizontalTextPosition(),
     viewRect, iconRect, textRect, textIconGap);

   // find union of icon and text rectangles
   Rectangle rect = iconRect.union (textRect);

   Insets insets = getInsets (c);

   rect.width += insets.left + insets.right;
   rect.height += insets.top + insets.bottom;

   return rect.getSize();
 }

 public Insets getDefaultMargin (AbstractButton b) {
   return defaultMargin;
 }


 public Insets getInsets (JComponent c) {
   Border border = c.getBorder();
   Insets insets = ((border != null) ?
     border.getBorderInsets (c) :
     new Insets (0,0,0,0));
   return insets;
 }
}

