package multipad;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.filechooser.*;


public class Multipad extends JFrame
{
  JDesktopPane    desktopPane;
  Container       content;
  JMenuBar        menuBar;
  JMenu           fileMenu, editMenu, formatMenu, subMenu, insertMenu, lnfMenu;
  JMenuItem       menuItem;
  JToolBar        toolBar;
  JButton         button;
  ImageIcon       menuIcons[] = new ImageIcon[12];
  ActionListener  al;


  public void init()
  {
    // ContentPane des Frames
    content = getContentPane();

    setTitle("Mulitpad");
    content.setLayout(new BorderLayout());

    // Desktop erzeugen
    desktopPane = new JDesktopPane();
    content.add("Center", desktopPane);

    // Icons einladen
    menuIcons[0] = new ImageIcon("new.gif");
    menuIcons[1] = new ImageIcon("open.gif");
    menuIcons[2] = new ImageIcon("save.gif");
    menuIcons[3] = new ImageIcon("cut.gif");
    menuIcons[4] = new ImageIcon("copy.gif");
    menuIcons[5] = new ImageIcon("paste.gif");
    menuIcons[6] = new ImageIcon("left.gif");
    menuIcons[7] = new ImageIcon("center.gif");
    menuIcons[8] = new ImageIcon("right.gif");
    menuIcons[9] = new ImageIcon("bold.gif");
    menuIcons[10] = new ImageIcon("italic.gif");
    menuIcons[11] = new ImageIcon("underline.gif");

    // Men aufbauen
    menuBar = new JMenuBar();
    toolBar = new JToolBar();
    toolBar.setFloatable(true);
    setJMenuBar(menuBar);
    content.add("North", toolBar);

    // Dateimen
    fileMenu = new JMenu("Datei");
    fileMenu.setMnemonic('D');

    fileMenu.add(menuItem = new JMenuItem("Neu...", menuIcons[0]));
    menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
    menuItem.setMnemonic('N');
    menuItem.setHorizontalTextPosition(JButton.RIGHT);
    toolBar.add(button = new JButton(menuIcons[0]));
    al = new ActionListener(){
      public void actionPerformed(ActionEvent e) { onFileNew("Unbekannt.pad"); }
    };
    menuItem.addActionListener(al);
    button.addActionListener(al);

    fileMenu.add(menuItem = new JMenuItem("ffnen...", menuIcons[1]));
    menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
    menuItem.setMnemonic('');
    menuItem.setHorizontalTextPosition(JButton.RIGHT);
    toolBar.add(button = new JButton(menuIcons[1]));
    al = new ActionListener(){
      public void actionPerformed(ActionEvent e) { onFileOpen(); }
    };
    menuItem.addActionListener(al);
    button.addActionListener(al);

    fileMenu.add(menuItem = new JMenuItem("Speichern", menuIcons[2]));
    menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
    menuItem.setMnemonic('S');
    menuItem.setHorizontalTextPosition(JButton.RIGHT);
    toolBar.add(button = new JButton(menuIcons[2]));
    al = new ActionListener(){
      public void actionPerformed(ActionEvent e){
        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        if (getActiveFrame()!=null)
          getActiveFrame().saveIt(getActiveFrame().getTitle());
        setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)); }
    };
    menuItem.addActionListener(al);
    button.addActionListener(al);

    fileMenu.add(menuItem = new JMenuItem("Speichern unter...", menuIcons[2]));
    menuItem.setMnemonic('u');
    menuItem.setHorizontalTextPosition(JButton.RIGHT);
    al = new ActionListener(){
      public void actionPerformed(ActionEvent e) { onFileSaveAs(); }
    };
    menuItem.addActionListener(al);

    fileMenu.addSeparator();
    toolBar.addSeparator();

    fileMenu.add(menuItem = new JMenuItem("Beenden"));
    al = new ActionListener(){
      public void actionPerformed(ActionEvent e) { System.exit(0); }
    };
    menuItem.setHorizontalTextPosition(JButton.RIGHT);
    menuItem.addActionListener(al);

    menuBar.add(fileMenu);


    // Editier menu
    editMenu = new JMenu("Bearbeiten");
    editMenu.setMnemonic('B');

    editMenu.add(menuItem = new JMenuItem("Ausschneiden", menuIcons[3]));
    menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, ActionEvent.CTRL_MASK));
    menuItem.setMnemonic('N');
    menuItem.setHorizontalTextPosition(JButton.RIGHT);
    toolBar.add(button = new JButton(menuIcons[3]));
    al = new ActionListener(){
      public void actionPerformed(ActionEvent e){
        getActiveFrame().getTextPane().cut(); }
    };
    menuItem.addActionListener(al);
    button.addActionListener(al);

    editMenu.add(menuItem = new JMenuItem("Kopieren", menuIcons[4]));
    menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK));
    menuItem.setMnemonic('K');
    menuItem.setHorizontalTextPosition(JButton.RIGHT);
    toolBar.add(button = new JButton(menuIcons[4]));
    al = new ActionListener(){
      public void actionPerformed(ActionEvent e){
        getActiveFrame().getTextPane().copy(); }
    };
    menuItem.addActionListener(al);
    button.addActionListener(al);

    editMenu.add(menuItem = new JMenuItem("Einfgen", menuIcons[5]));
    menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, ActionEvent.CTRL_MASK));
    menuItem.setMnemonic('E');
    menuItem.setHorizontalTextPosition(JButton.RIGHT);
    toolBar.add(button = new JButton(menuIcons[5]));
    al = new ActionListener(){
      public void actionPerformed(ActionEvent e){
        getActiveFrame().getTextPane().paste(); }
    };
    menuItem.addActionListener(al);
    button.addActionListener(al);
    toolBar.addSeparator();
    menuBar.add(editMenu);


    // Formatierung
    menuBar.add(formatMenu = new JMenu("Formatieren"));
    formatMenu.setMnemonic('F');

    formatMenu.add(subMenu = new JMenu("Absatz"));
    subMenu.add(menuItem = new JMenuItem("Links"));
    toolBar.add(button = new JButton(menuIcons[6]));
    al = new StyledEditorKit.AlignmentAction("alignment-left", StyleConstants.ALIGN_LEFT);
    menuItem.addActionListener(al);
    button.addActionListener(al);

    subMenu.add(menuItem = new JMenuItem("Mitte"));
    toolBar.add(button = new JButton(menuIcons[7]));
    al = new StyledEditorKit.AlignmentAction("alignment-center", StyleConstants.ALIGN_CENTER);
    menuItem.addActionListener(al);
    button.addActionListener(al);

    subMenu.add(menuItem = new JMenuItem("Rechts"));
    toolBar.add(button = new JButton(menuIcons[8]));
    al = new StyledEditorKit.AlignmentAction("alignment-right", StyleConstants.ALIGN_RIGHT);
    menuItem.addActionListener(al);
    button.addActionListener(al);

    toolBar.addSeparator();

    formatMenu.add(subMenu = new JMenu("Zeichen"));
    subMenu.add(menuItem = new JMenuItem("Fett"));
    menuItem.setFont(new Font("Dialog", Font.BOLD, 12));
    toolBar.add(button = new JButton(menuIcons[9]));
    al = new StyledEditorKit.BoldAction();
    menuItem.addActionListener(al);
    button.addActionListener(al);

    subMenu.add(menuItem = new JMenuItem("Kursiv"));
    menuItem.setFont(new Font("Dialog", Font.ITALIC, 12));
    toolBar.add(button = new JButton(menuIcons[10]));
    al = new StyledEditorKit.ItalicAction();
    menuItem.addActionListener(al);
    button.addActionListener(al);

    subMenu.add(menuItem = new JMenuItem("Unterstrichen"));
    toolBar.add(button = new JButton(menuIcons[11]));
    al = new StyledEditorKit.UnderlineAction();
    menuItem.addActionListener(al);
    button.addActionListener(al);

    formatMenu.add(subMenu = new JMenu("Schriftart"));
    String[]  fontList = getToolkit().getFontList();
    for (int i=0; i<fontList.length; i++)
    {
      subMenu.add(menuItem = new JMenuItem(fontList[i]));
      menuItem.setFont(new Font(fontList[i], Font.PLAIN, 12));
      menuItem.addActionListener(new StyledEditorKit.FontFamilyAction("font-family-" + fontList[i], fontList[i]));
    }

    formatMenu.add(subMenu = new JMenu("Schriftgre"));
    for (int i=8; i<=26; i+=2)
    {
      subMenu.add(menuItem = new JMenuItem(Integer.toString(i)));
      menuItem.addActionListener(new StyledEditorKit.FontSizeAction("font-size-" + Integer.toString(i), i));
    }

    formatMenu.add(subMenu = new JMenu("Farbe"));
    subMenu.add(menuItem = new JMenuItem(new ColorField(Color.black)));
    menuItem.addActionListener(new StyledEditorKit.ForegroundAction("foreground-black", Color.black));

    subMenu.add(menuItem = new JMenuItem(new ColorField(Color.white)));
    menuItem.addActionListener(new StyledEditorKit.ForegroundAction("foreground-white", Color.white));

    subMenu.add(menuItem = new JMenuItem(new ColorField(Color.gray)));
    menuItem.addActionListener(new StyledEditorKit.ForegroundAction("foreground-gray", Color.gray));

    subMenu.add(menuItem = new JMenuItem(new ColorField(Color.red)));
    menuItem.addActionListener(new StyledEditorKit.ForegroundAction("foreground-red", Color.red));

    subMenu.add(menuItem = new JMenuItem(new ColorField(Color.pink)));
    menuItem.addActionListener(new StyledEditorKit.ForegroundAction("foreground-pink", Color.pink));

    subMenu.add(menuItem = new JMenuItem(new ColorField(Color.blue)));
    menuItem.addActionListener(new StyledEditorKit.ForegroundAction("foreground-blue", Color.blue));

    subMenu.add(menuItem = new JMenuItem(new ColorField(Color.green)));
    menuItem.addActionListener(new StyledEditorKit.ForegroundAction("foreground-green", Color.green));


    // Men: Einfgen
    menuBar.add(insertMenu = new JMenu("Einfgen"));
    insertMenu.setMnemonic('E');

    insertMenu.add(menuItem = new JMenuItem("Textdatei"));
    al = new ActionListener(){
      public void actionPerformed(ActionEvent e) { onInsertTextFile(); }
    };
    menuItem.addActionListener(al);

    insertMenu.add(menuItem = new JMenuItem("Bild"));
    al = new ActionListener(){
      public void actionPerformed(ActionEvent e) { onInsertImage(); }
    };
    menuItem.addActionListener(al);


    // Men: Look & Feel
    menuBar.add(lnfMenu = new JMenu("Look & Feel"));
    insertMenu.setMnemonic('L');

    ButtonGroup lnfGroup = new ButtonGroup();
    JRadioButtonMenuItem  lnfItem;
    lnfMenu.add(lnfItem = new JRadioButtonMenuItem("Metal"));
    lnfGroup.add(lnfItem);
    lnfItem.setSelected(true);
    al = new ActionListener(){
      public void actionPerformed(ActionEvent e) {
        setLookAndFeel(e.getActionCommand()); }
    };
    lnfItem.addActionListener(al);

    lnfMenu.add(lnfItem = new JRadioButtonMenuItem("Motif"));
    lnfGroup.add(lnfItem);
    lnfItem.addActionListener(al);

    lnfMenu.add(lnfItem = new JRadioButtonMenuItem("Windows"));
    lnfGroup.add(lnfItem);
    lnfItem.addActionListener(al);
  }

  public void onFileNew(String fileName)
  {
    System.out.println("Neue Datei");
    MultipadFrame frame;
    desktopPane.add(frame = new MultipadFrame(fileName), JLayeredPane.DEFAULT_LAYER);
    frame.moveToFront();
    frame.setVisible(true);
    try {
      frame.setSelected(true);
    }
    catch(Exception e) { }
  }

  public void onFileOpen()
  {
    System.out.println("Datei ffnen");
    JFileChooser fileChooser = new JFileChooser();
    fileChooser.addChoosableFileFilter(new CustomFileFilter("pad", "Multipad Dokumente"));
    fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
    fileChooser.setMultiSelectionEnabled(false);
    fileChooser.setDialogTitle("Dokument laden...");
    if (fileChooser.showOpenDialog(this)==JFileChooser.APPROVE_OPTION)
    {
      setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
      onFileNew("");
      getActiveFrame().loadIt(fileChooser.getSelectedFile().toString());
      setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }
  }


  public void onFileSaveAs()
  {
    if (getActiveFrame() == null)
      return;
    JFileChooser fileChooser = new JFileChooser();
    fileChooser.addChoosableFileFilter(new CustomFileFilter("pad", "Multipad Dokumente"));
    fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
    fileChooser.setMultiSelectionEnabled(false);
    fileChooser.setDialogTitle("Dokument speichern unter...");
    if (fileChooser.showSaveDialog(this)==JFileChooser.APPROVE_OPTION)
    {
      setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
      getActiveFrame().saveIt(fileChooser.getSelectedFile().toString());
      setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }
  }


  public void onInsertImage()
  {
    if (getActiveFrame() == null)
      return;
    JFileChooser fileChooser = new JFileChooser();
    fileChooser.addChoosableFileFilter(new CustomFileFilter("gif", "GIF-Bilder"));
    fileChooser.addChoosableFileFilter(new CustomFileFilter("jpg", "JPG-Bilder"));
    fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
    fileChooser.setMultiSelectionEnabled(false);
    fileChooser.setDialogTitle("Bild einfgen...");
    if (fileChooser.showOpenDialog(this)==JFileChooser.APPROVE_OPTION)
    {
      setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
      getActiveFrame().insertImage(fileChooser.getSelectedFile().toString());
      setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }
  }


  public void onInsertTextFile()
  {
    if (getActiveFrame() == null)
      return;
    JFileChooser fileChooser = new JFileChooser();
    fileChooser.addChoosableFileFilter(new CustomFileFilter("txt", "ASCII-Textdateien"));
    fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
    fileChooser.setMultiSelectionEnabled(false);
    fileChooser.setDialogTitle("Textdatei einfgen...");
    if (fileChooser.showOpenDialog(this)==JFileChooser.APPROVE_OPTION)
    {
      setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
      getActiveFrame().insertTextFile(fileChooser.getSelectedFile().toString());
      setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }
  }

  public MultipadFrame getActiveFrame()
  {
    JInternalFrame  allFrames[] = desktopPane.getAllFrames();

    for (int i=0; i<allFrames.length; i++)
      if (allFrames[i].isSelected() == true)
        return (MultipadFrame)allFrames[i];

    return null;
  }

  public void setLookAndFeel(String lnf)
  {
    String lnfName;

    lnfName = "javax.swing.plaf.metal.MetalLookAndFeel";
    if (lnf.equals("Motif"))
      lnfName = "com.sun.java.swing.plaf.motif.MotifLookAndFeel";
    if (lnf.equals("Windows"))
      lnfName = "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
    try
    {
      UIManager.setLookAndFeel(lnfName);
      SwingUtilities.updateComponentTreeUI(this);
    }
    catch (Exception exc) {
      System.err.println("Konnte das Look&Feel nicht laden: " + lnfName);
    }
  }

  public static void main(String[] args)
  {
    Multipad mulitPad = new Multipad();

    WindowListener l = new WindowAdapter() {
        public void windowClosing(WindowEvent e) {System.exit(0);}
    };
    mulitPad.addWindowListener(l);

    mulitPad.init();
    mulitPad.setSize(800, 600);
    mulitPad.show();
  }
}



class MultipadFrame extends JInternalFrame
{
  private JTextPane  textPane;
  StyledDocument frameDocument;


  MultipadFrame(String title)
  {
    super(title, true, true, true, true);
    setBounds(50, 20, 380, 250);

    textPane = new JTextPane();
    textPane.setStyledDocument(frameDocument = new DefaultStyledDocument());
    JScrollPane scrollPane = new JScrollPane(textPane);
    scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
    scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    setContentPane(scrollPane);
    validate();
  }


  public JTextPane getTextPane()
  {
    return textPane;
  }

  public StyledDocument getDocument()
  {
    return frameDocument;
  }

  public void loadIt(String fileName)
  {
    File  f = new File(fileName);
    try {
      FileInputStream fstrm = new FileInputStream(f);
      ObjectInput istrm = new ObjectInputStream(fstrm);
      frameDocument = (StyledDocument) istrm.readObject();
      Rectangle bounds = (Rectangle) istrm.readObject();
      textPane.setStyledDocument(frameDocument);
      validate();
      setBounds(bounds);
      istrm.close();
    }
    catch (IOException io) {
      System.err.println("Loadefehler: " + io.getMessage());
    } catch (ClassNotFoundException cnf) {
      System.err.println("Formatfehler: " + cnf.getMessage());
    }
    setTitle(fileName);
  }

  public void saveIt(String fileName)
  {
    if (!fileName.substring(fileName.length()-4).equals(".pad"))
      fileName = fileName + ".pad";
    File  f = new File(fileName);
    try {
      FileOutputStream fstrm = new FileOutputStream(f);
      ObjectOutput ostrm = new ObjectOutputStream(fstrm);
      ostrm.writeObject(frameDocument);
      ostrm.writeObject(getBounds());
      ostrm.flush();
      ostrm.close();
    }
    catch (IOException io) {
      System.err.println("Fehler beim Speichern: " + io.getMessage());
    }
    setTitle(fileName);
  }

  public void insertImage(String fileName)
  {
    ImageIcon  image = new ImageIcon(fileName);
    if (image!=null)
      textPane.insertIcon(image);
  }

  public void insertTextFile(String fileName)
  {
    StyledDocument doc = textPane.getStyledDocument();

    try {
      String s = loadTextFile(fileName);
      doc.insertString(textPane.getCaretPosition(), s, null);
      validate();
    }
    catch (BadLocationException e) {
      System.out.println("Ungltige Position: " + e.getMessage());
    }
  }


  // Ldt eine Textdatei ein
  public String loadTextFile(String fileName)
  {
    File  file = new File(fileName);
    String s = new String();
    try
    {
      int nch;
      char[] buff = new char[50000];
      FileReader fr = new FileReader(file);
      BufferedReader reader = new BufferedReader(fr);
      while ((nch = reader.read(buff, 0, buff.length)) != -1)
         s = s + new String(buff, 0, nch);
    }
    catch (java.io.IOException ex)
    {
      s = "Datei konnte nicht geladen werden: " + fileName;
    }

    return s;
  }
}


class CustomFileFilter extends javax.swing.filechooser.FileFilter
{
  String  extension, description;

  CustomFileFilter(String ex, String dsc)
  {
    super();
    extension = ex;
    description = dsc;
  }

  public boolean accept(File file)
  {
    if (file.isDirectory())
      return true;

    String fileName = file.getName();
    int i = fileName.lastIndexOf('.');

    if (i>0 &&  i<fileName.length() - 1)
    {
      String ext = fileName.substring(i+1).toLowerCase();
      if (ext.equals(extension))
        return true;
      else
        return false;
    }

    return false;
  }

  public String getDescription()
  {
    return description;
  }
}


class ColorField implements Icon
{
  Color color;

  public ColorField(Color c)
  {
      this.color = c;
  }

  public void paintIcon(Component c, Graphics g, int x, int y)
  {
      g.setColor(color);
      g.fill3DRect(x,y,getIconWidth(), getIconHeight(), true);
  }
  public int getIconWidth() { return 40; }
  public int getIconHeight() { return 12; }
}