package tabdemo;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

public class TabDemo extends JFrame {
	public static final int LIST_TAB = 0;
	public static final int DATA_TAB = 1;

	protected Person[] m_employees = {
		new Person("Hans", "Mller", "01-9481111"),
		new Person("Silvia", "Heinrichs", "031-4563436"),
		new Person("Captain", "Kirk", "0002-001"),
		new Person("Duke", "Nukem", "00003-101010"),
		new Person("James", "Bond", "007-007")
	};

	protected JList      m_list;
	protected JTextField m_firstTxt;
	protected JTextField m_lastTxt;
	protected JTextField m_phoneTxt;
	protected JTabbedPane m_tab;

	public TabDemo() {
		super("Tab Demo");

		JPanel p1 = new JPanel(new BorderLayout());
		p1.setBorder(new EmptyBorder(10, 10, 10, 10));
		m_list = new JList(m_employees);
		m_list.setVisibleRowCount(4);
		JScrollPane sp = new JScrollPane(m_list);
		p1.add(sp, BorderLayout.CENTER);

		MouseListener mlst = new MouseAdapter() {
			public void mouseClicked(MouseEvent evt) {
				if (evt.getClickCount() == 2)
					m_tab.setSelectedIndex(DATA_TAB);
			}
		};
		m_list.addMouseListener(mlst);

		JPanel p2 = new JPanel(new DialogLayout());
		p2.setBorder(new EmptyBorder(10, 10, 10, 10));
		p2.add(new JLabel("Vorname:"));
		m_firstTxt = new JTextField(20);
		p2.add(m_firstTxt);
		p2.add(new JLabel("Nachname:"));
		m_lastTxt = new JTextField(20);
		p2.add(m_lastTxt);
		p2.add(new JLabel("Telefonnummer:"));
		m_phoneTxt = new JTextField(20);
		p2.add(m_phoneTxt);

		m_tab = new JTabbedPane();
		m_tab.addTab("Mitarbeiter", p1);
		m_tab.addTab("Personal Daten", p2);
		m_tab.addChangeListener(new TabChangeListener());

		JPanel p = new JPanel();
		p.add(m_tab);
		p.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(p);
		pack();
	}

	public Person getSelectedPerson() {
		return (Person)m_list.getSelectedValue();
	}

	public static void main(String[] args) {
		JFrame frame = new TabDemo();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setVisible(true);
	}

	class TabChangeListener implements ChangeListener {
		public void stateChanged(ChangeEvent e) {
			Person sp = getSelectedPerson();
			switch(m_tab.getSelectedIndex())
			{
			case DATA_TAB:
				if (sp == null) {
					m_tab.setSelectedIndex(LIST_TAB);
					return;
				}
				m_firstTxt.setText(sp.m_firstName);
				m_lastTxt.setText(sp.m_lastName);
				m_phoneTxt.setText(sp.m_phone);
				break;

			case LIST_TAB:
				if (sp != null) {
					sp.m_firstName = m_firstTxt.getText();
					sp.m_lastName = m_lastTxt.getText();
					sp.m_phone = m_phoneTxt.getText();
					m_list.repaint();
				}
				break;
			}
		}
	}

	class Person {
		public String m_firstName;
		public String m_lastName;
		public String m_phone;

		public Person(String firstName, String lastName, String phone) {
			m_firstName = firstName;
			m_lastName = lastName;
			m_phone = phone;
		}

		public String toString() {
			String str = m_firstName+" "+m_lastName;
			if (m_phone.length() > 0)
				str += " ("+m_phone+")";
			return str.trim();
		}
	}
}
