package fontsfileshandlingmenubar3;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

public class BasicTextEditor
	extends JFrame {

	public static final String APP_NAME = "Basic Text Editor";

	public static final String FONTS[] = { "Serif", "SansSerif",
		"Courier" };
	protected Font m_fonts[];

	protected JTextArea m_editor;
	protected JMenuItem[] m_fontMenus;
	protected JCheckBoxMenuItem m_bold;
	protected JCheckBoxMenuItem m_italic;

	protected JFileChooser m_chooser;
	protected File  m_currentFile;

	protected JToolBar m_toolBar;
    protected JComboBox m_cbFonts;	// neu
    protected SmallToggleButton m_bBold;
    protected SmallToggleButton m_bItalic;

	public BasicTextEditor() {
		super(APP_NAME+": Teil III - Custom Toolbar Buttons");
		setSize(450, 350);

		m_fonts = new Font[FONTS.length];
		for (int k=0; k<FONTS.length; k++)
			m_fonts[k] = new Font(FONTS[k], Font.PLAIN, 12);

		m_editor = new JTextArea();
		JScrollPane ps = new JScrollPane(m_editor);
		getContentPane().add(ps, BorderLayout.CENTER);

		m_editor.append(APP_NAME);

		JMenuBar menuBar = createMenuBar();
		setJMenuBar(menuBar);

		m_chooser = new JFileChooser();
		m_chooser.setCurrentDirectory(new File("."));

		updateEditor();
	}

	protected JMenuBar createMenuBar() {
		final JMenuBar menuBar = new JMenuBar();

		JMenu mFile = new JMenu("Datei");
		mFile.setMnemonic('f');

		ImageIcon iconNew = new ImageIcon("New16.gif");
		Action actionNew = new AbstractAction("Neu", iconNew) {
			public void actionPerformed(ActionEvent e) {
				m_editor.setText("");
				m_currentFile = null;
				setTitle(APP_NAME+" [neueTextdatei.txt]");
			}
		};

		JMenuItem item = new JMenuItem(actionNew);
		item.setIcon(new ImageIcon("New16.gif"));
		item.setMnemonic('n');
		item.setAccelerator(KeyStroke.getKeyStroke(
			KeyEvent.VK_N, InputEvent.CTRL_MASK));
		mFile.add(item);


		ImageIcon iconOpen = new ImageIcon("Open16.gif");
		Action actionOpen = new AbstractAction("Oeffnen...", iconOpen) {
			public void actionPerformed(ActionEvent e) {
				int result = m_chooser.showOpenDialog(BasicTextEditor.this);
				repaint();	//alter Fehler
				if (result != JFileChooser.APPROVE_OPTION)
					return;
			    File f = m_chooser.getSelectedFile();
			    if (f == null || !f.isFile())
					return;
			    m_currentFile = f;
				try {
					FileReader in = new FileReader(m_currentFile);
					m_editor.read(in, null);
					in.close();
					setTitle(APP_NAME+" ["+m_currentFile.getName()+"]");
				}
				catch (IOException ex) {
					showError(ex, "Fehler beim Lesen der Datei "+m_currentFile);
				}
			}
		};

		item = new JMenuItem(actionOpen);
		item.setMnemonic('o');
		item.setAccelerator(KeyStroke.getKeyStroke(
			KeyEvent.VK_O, InputEvent.CTRL_MASK));
		mFile.add(item);

		ImageIcon iconSave = new ImageIcon("Save16.gif");
		Action actionSave = new AbstractAction("Speichern...", iconSave) {
			public void actionPerformed(ActionEvent e) {
				if (m_currentFile == null) {
					int result = m_chooser.showSaveDialog(BasicTextEditor.this);
					repaint();
					if (result != JFileChooser.APPROVE_OPTION)
						return;
					File f = m_chooser.getSelectedFile();
					if (f == null)
						return;
					m_currentFile = f;
					setTitle(APP_NAME+" ["+m_currentFile.getName()+"]");
				}

				try {
					FileWriter out = new
						FileWriter(m_currentFile);
					m_editor.write(out);
					out.close();
				}
				catch (IOException ex) {
					showError(ex, "Fehler beim Lesen der Datei "+m_currentFile);
				}
			}
		};

		item = new JMenuItem(actionSave);
		item.setMnemonic('s');
		item.setAccelerator(KeyStroke.getKeyStroke(
			KeyEvent.VK_S, InputEvent.CTRL_MASK));
		mFile.add(item);

		mFile.addSeparator();

		Action actionExit = new AbstractAction("Exit") {
			public void actionPerformed(ActionEvent e) {
				System.exit(0);
			}
		};
		item = new JMenuItem(actionExit);
		item.setMnemonic('x');
		mFile.add(item);
		menuBar.add(mFile);

		// Create toolbar
		m_toolBar = new JToolBar();
		JButton bNew = new SmallButton(actionNew,
			"Neuer Text");
		m_toolBar.add(bNew);
		JButton bOpen = new SmallButton(actionOpen,
			"Oeffnen einer Textdatei");
		m_toolBar.add(bOpen);
		JButton bSave = new SmallButton(actionSave,
			"Speichern in einer Textdatei");
		m_toolBar.add(bSave);
		getContentPane().add(m_toolBar, BorderLayout.NORTH);

		ActionListener fontListener = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateEditor();
			}
		};

		JMenu mFont = new JMenu("Font");
		mFont.setMnemonic('o');

		ButtonGroup group = new ButtonGroup();
		m_fontMenus = new JMenuItem[FONTS.length];
		for (int k=0; k<FONTS.length; k++) {
			int m = k+1;
			m_fontMenus[k] = new JRadioButtonMenuItem(
				m+" "+FONTS[k]);
			m_fontMenus[k].setSelected(k == 0);
			m_fontMenus[k].setMnemonic('1'+k);
			m_fontMenus[k].setFont(m_fonts[k]);
			m_fontMenus[k].addActionListener(fontListener);
			group.add(m_fontMenus[k]);
			mFont.add(m_fontMenus[k]);
		}

		mFont.addSeparator();

		// Add combobox to tollbar - NEW
		m_toolBar.addSeparator();
		m_cbFonts = new JComboBox(FONTS);
		m_cbFonts.setMaximumSize(m_cbFonts.getPreferredSize());
		m_cbFonts.setToolTipText("Verfgbare Fonts");
		ActionListener lst = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int index = m_cbFonts.getSelectedIndex();
				if (index < 0)
					return;
				m_fontMenus[index].setSelected(true);
				updateEditor();
			}
		};
		m_cbFonts.addActionListener(lst);
		m_toolBar.add(m_cbFonts);

		m_toolBar.addSeparator();
		m_bold = new JCheckBoxMenuItem("Fett");
		m_bold.setMnemonic('b');
		Font fn = m_fonts[1].deriveFont(Font.BOLD);
		m_bold.setFont(fn);
		m_bold.setSelected(false);
		m_bold.addActionListener(fontListener);
		mFont.add(m_bold);

		m_italic = new JCheckBoxMenuItem("Kursiv");
		m_italic.setMnemonic('i');
		fn = m_fonts[1].deriveFont(Font.ITALIC);
		m_italic.setFont(fn);
		m_italic.setSelected(false);
		m_italic.addActionListener(fontListener);
		mFont.add(m_italic);

		menuBar.add(mFont);

		ImageIcon img1 = new ImageIcon("Bold16.gif");
		m_bBold = new SmallToggleButton(false, img1, img1,
			"fett");
		lst = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				m_bold.setSelected(m_bBold.isSelected());
				updateEditor();
			}
		};
		m_bBold.addActionListener(lst);
		m_toolBar.add(m_bBold);

		img1 = new ImageIcon("Italic16.gif");
		m_bItalic = new SmallToggleButton(false, img1, img1,
			"kursiv");
		lst = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				m_italic.setSelected(m_bItalic.isSelected());
				updateEditor();
			}
		};
		m_bItalic.addActionListener(lst);
		m_toolBar.add(m_bItalic);

		return menuBar;
	}

	protected void updateEditor() {
		int index = -1;
		for (int k=0; k<m_fontMenus.length; k++) {
			if (m_fontMenus[k].isSelected()) {
				index = k;
				break;
			}
		}
		if (index == -1)
			return;

		if (index==2) { // Courier
			m_bold.setSelected(false);
			m_bold.setEnabled(false);
			m_italic.setSelected(false);
			m_italic.setEnabled(false);
			m_bBold.setSelected(false);
			m_bBold.setEnabled(false);
			m_bItalic.setSelected(false);
			m_bItalic.setEnabled(false);
		}
		else {
			m_bold.setEnabled(true);
			m_italic.setEnabled(true);
			m_bBold.setEnabled(true);
			m_bItalic.setEnabled(true);
		}

		// Synchronize toolbar and menu components
		m_cbFonts.setSelectedIndex(index);
		boolean isBold = m_bold.isSelected();
		boolean isItalic = m_italic.isSelected();
		if (m_bBold.isSelected() != isBold)
			m_bBold.setSelected(isBold);
		if (m_bItalic.isSelected() != isItalic)
			m_bItalic.setSelected(isItalic);

		int style = Font.PLAIN;
		if (m_bold.isSelected())
			style |= Font.BOLD;
		if (m_italic.isSelected())
			style |= Font.ITALIC;
		Font fn = m_fonts[index].deriveFont(style);
		m_editor.setFont(fn);
		m_editor.repaint();
	}

	public void showError(Exception ex, String message) {
		ex.printStackTrace();
		JOptionPane.showMessageDialog(this,
			message, APP_NAME,
			JOptionPane.WARNING_MESSAGE);
	}

	public static void main(String argv[]) {
		BasicTextEditor frame = new BasicTextEditor();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setVisible(true);
	}
}

class SmallButton extends JButton implements MouseListener {
	protected Border m_raised;
	protected Border m_lowered;
	protected Border m_inactive;

	public SmallButton(Action act, String tip) {
		super((Icon)act.getValue(Action.SMALL_ICON));
		m_raised = new SoftBevelBorder(BevelBorder.RAISED);
		m_lowered = new SoftBevelBorder(BevelBorder.LOWERED);
		m_inactive = new EmptyBorder(3, 3, 3, 3);
		setBorder(m_inactive);
		setMargin(new Insets(1,1,1,1));
		setToolTipText(tip);
		addActionListener(act);
		addMouseListener(this);
		setRequestFocusEnabled(false);
	}

	public float getAlignmentY() {
		return 0.5f;
	}

	public void mousePressed(MouseEvent e) {
		setBorder(m_lowered);
	}

	public void mouseReleased(MouseEvent e) {
		setBorder(m_inactive);
	}

	public void mouseClicked(MouseEvent e) {}

	public void mouseEntered(MouseEvent e) {
		setBorder(m_raised);
	}

	public void mouseExited(MouseEvent e) {
		setBorder(m_inactive);
	}
}

class SmallToggleButton extends JToggleButton
	implements ItemListener {

	protected Border m_raised;
	protected Border m_lowered;

	public SmallToggleButton(boolean selected,
		ImageIcon imgUnselected, ImageIcon imgSelected, String tip) {
		super(imgUnselected, selected);
		setHorizontalAlignment(CENTER);
		setBorderPainted(true);
		m_raised = new SoftBevelBorder(BevelBorder.RAISED);
		m_lowered = new SoftBevelBorder(BevelBorder.LOWERED);
		setBorder(selected ? m_lowered : m_raised);
		setMargin(new Insets(1,1,1,1));
		setToolTipText(tip);
		setRequestFocusEnabled(false);
		setSelectedIcon(imgSelected);
		addItemListener(this);
	}

	public float getAlignmentY() {
		return 0.5f;
	}

	public void itemStateChanged(ItemEvent e) {
		setBorder(isSelected() ? m_lowered : m_raised);
	}
}
